//modified:2003.01.20@21:35 by Alex
//Project: TI-Server v3

//handles input from and output to ti

unit TIobject;

interface

uses
  Forms, Classes, SysUtils, HttpProt, SmtpProt, Pop3Prot, UnitTypes,
  Dialogs, Math;

type
  TProcess  = class
  public
    // 1. Nutzereigenschaften
    myaddr         : integer;
    myuser         : string;
    mytitype       : TTIType; // type of TI
    loggedin       : boolean; // whether user has logged in
    logtext   : string;
    packets   : array of TPacket;
    userdatai : integer; // index of this user in userdatai
    // 2. Variablen, die zur Navigation gebraucht werden
    actions   : array of TAction;
    longtext  : string;  // stores plain text to be shown in parts (navigation with TI's left and right key)
    longtextn : integer; // number of pages of the current long text
    longtextp : integer; // page no. of longtext we have sent to the TI lately (each page contains 7 lines, um 1 Zeile berlappen sich die Seiten, first page=0)
    longmenu  : TLongMenu;//stores a long menu
    storeto   : string;  // name of variable to which we save the next input
    okAct     : Taction; // action to commit when user quits input by hitting ok
    breakAct  : Taction; // action to commit when user quits input by hitting cancel
    ack       : boolean; // send an acknowledge-packet as soon as possible (-> TProcess.GetPacket)
    triggerunregister : boolean;
    lastcfile : string;  // last content file sent to ti
    shownmsg  : integer; // currently shown message
    shownmail : integer; // currently shown mail
    shownuser : integer; // currently edited user
    expectcid : byte;    // expect the following cid as reply from UU
    rcvdempty : boolean; // received an empty packet
    // 2.1 Variablen fr Multipacket
    mpbuffer  : string;  // buffer used when receiving multi packets to temp.ly store received data (here: multi packet data received from TI)
    mpnop     : integer; // number of packets in current multi packet
    mplastrcvd: byte;    // holds number of last multipacket part-packet
    // 3. Variablen, in denen Benutzereingaben zwischengespeichert werden
    datahash  : THash;
    // 3.1 Mail
//  emailtext : string;
//  emailrcpt : string;
//  emailsubj : string;
    // 3.2 SMS
//  smstext   : string;
//  smsnummer : string;
//  smsthread : TThreadSMS;
    // 3.3 Ti-Net message
//  msgtext   : string;
//  msgrcpt   : string;
    // 3.4 Sonstiges
//  password  : string;  // password for login
//  password2 : string;  // password2 for changing password
//  temp      : string;  // alles Mgliche, wird immer als Leerstring ausgelesen
    // 4. Variablen fr Maileinstellungen
    MsgHeader : boolean; //dient zum berspringen des MessageHeaders
    mailnum   : integer; //Nummer der gerade empfangenen Mail
    mails     : array of TMail;//hier werden die empfangenen Mails gespeichert
    // 5. Sonstiges
    noreplycount : byte; // counts number of the packets in a row the client has not replied to
    resent    : integer; // counts how often a packet had to be resent
    maxresent : integer; // maximum of resent
    // 6. Prozeduren/Funktionen
    constructor Create(addr:integer;user:string;titype:integer);
    destructor  Free;
    function    GetAddr:integer;    //diese umstndliche Handhabung (Variablen werden als private deklariert und nur ber Funktionen rausgegeben, kann man sicher auch sparen)
    function    GetUserName:string; // "
    procedure   AddPacket(lid:byte;ldata:string);
    function    GetPacket:TPacket;
    procedure   ProcessPacket(lpacket:string);
    procedure   Log(text:string);
    procedure   ParseActions(action:Taction);
    procedure   Send2TIMainMenu;
    procedure   Send2TIContentFile(filename:string);
    function    LoadFileToStr(var cfile:TAoS;filename:string):boolean;
    function    MakeAction(fkeycode:integer;ffilename:string):Taction;
    procedure   SendMail;
    procedure   SMTPRequestDone(Sender: TObject; RqType: TSmtpRequest; Error: Word);
    procedure   GetMail;
    procedure   Pop3RequestDone(Sender: TObject; RqType: TPop3Request; Error: Word);
    procedure   Pop3ListLine(Sender: TObject);
    procedure   Pop3MessageLine(Sender: TObject);
    function    GetRandomMOTD:string;
    function    UnregisterMeNow:boolean;
    procedure   StoreToVar(data:string);
    function    LoadFromVar(data:string):string;
    procedure   Send2TIServerError(text:string);
    procedure   Send2TIServerMessage(text:string;nextstep:string);
    procedure   Send2TILongText(text:String);
    procedure   Send2TILongTextPage(pageno:integer);
    procedure   Send2TILongMenu(menutitle:string;default:Integer);
    procedure   Send2TILongMenuPage(pageno:integer);
    procedure   Send2TILogInScreen;
    function    TextASC2TI(text:string):string;
    function    TextTI2ASC(text:string):string;
    function    WrapText(text:string):string;
    function    Translate(var word:string;var translation:string;direction:Translate):boolean;
    function    UserExists(name:string):boolean;
  end;

implementation

uses
  Unit1,UnitConfig;


                                { TProcess }


constructor TProcess.Create(addr:integer;user:string;titype:integer);
//create new thread with username, calc type and addr info
var
  i      : integer;
  found  : boolean;
  ltitype: TTiType;
begin
  myaddr    := addr;
  for ltitype:=low(TILoginCode) to high(TILogInCode) do
    if TILoginCode[ltitype]=titype then mytitype := ltitype;
  shownmsg  := -1;
  lastcfile := '';
  resent    := 0;
  datahash  := THash.Create;
  if pos(#0,user)=0 then
    myuser:=user
  else
    myuser:=copy(user,1,pos(#0,user)-1);
  found :=false;
  //queue login ack as first packet
  SetLength(packets,0);
  AddPacket(cidloginack,myuser+StringOfChar(#0,8-length(myuser))+chr(myaddr));
  for i:=0 to high(userdata) do
    if userdata[i].uname = uppercase(myuser) then begin
      found        :=true;
      userdatai    :=i;
      break;
    end;
  if not found then begin
    Send2TIServerMessage('Dein Nutzername existiert nicht.','IDlogout');
  end else begin
    if userdata[i].online then
      Send2TIServerMessage(myuser+' ist bereits online.','IDlogout')
    else begin // username accepted
      triggerunregister:=false;
      userdata[i].online:=true;
      if Form2.CheckCSLoginCompulsory.Checked then
        Send2TILoginScreen
      else
        Send2TIMainMenu;  // skip login
    end;
  end;
  inherited Create;
end;


destructor TProcess.Free;
//nothing to do
begin
  //
end;


procedure TProcess.AddPacket(lid:byte;ldata:string);
//add a packet to queue, split into multipacket if necessary
var
  nop : integer; //number of packets
  i   : integer;
begin
  nop := ((length(ldata)-1) div Form1.maxpacketlen)+1;
  //evaluate which content id the uu will send as reply
  case lid of
    cidplaintext:        expectcid:=cidreplyplaintext;
    cidfullscreenmenu:   expectcid:=cidreplyfullscreenmenu;
    cidtiml:             expectcid:=cidreplytiml;
    cidscreenshot:       expectcid:=cidreplyscreenshot;
    cideditfield:        expectcid:=cidreplyeditfield;
    else                 expectcid:=0;
  end;
  if nop=1 then begin
    //single packet sufficient
    SetLength(packets,length(packets)+1);
    with packets[high(packets)] do begin
      id    :=lid;
      mpinfo:=1;
      data  :=ldata;
    end;
  end else begin
    //multi packet
    if nop>255 then begin
      nop:=255;
      Log('Multi-Packet htte mehr als 255 Teilpakete! Daten:'+ldata);
    end;
    SetLength(packets,length(packets)+1);
    with packets[high(packets)] do begin
      id    :=lid;
      mpinfo:=nop;
      data  :=copy(ldata,1,Form1.maxpacketlen);
    end;
    for i:=2 to nop do begin
      SetLength(packets,length(packets)+1);
      with packets[high(packets)] do begin
        id    :=lid or 128;
        mpinfo:=i;
        data  :=copy(ldata,(i-1)*Form1.maxpacketlen+1,Form1.maxpacketlen);
      end;
    end;
  end;
end;


function TProcess.GetPacket:TPacket;
begin
  if ack then begin //send ack
    Result.id:=cidack;
    Result.data:='';
    ack:=false;
    exit;
  end;
  if length(packets)>0 then
    if ((ord(packets[0].id) in cidsetcontrolServer2TI) or (rcvdempty)) then begin
      Result :=packets[0];
      inc(resent);
      if resent>maxresent then maxresent:=resent;
      exit;
    end;
  //send empty packet
  Result.id:=cidemptypacket;
  Result.data:='';
  resent:=0;
end;


function TProcess.GetAddr:integer;
//returns myaddr
begin
  result:=myaddr;
end;


function TProcess.GetUserName:string;
//returns myuser
begin
  result:=myuser;
end;


function TProcess.UnregisterMeNow:boolean;
begin
  result:=triggerunregister;
end;


////////////////////////////////////////////////////////////////////////////////
//
//             P R O C E S S   P A C K E T   F R O M   T I
//
procedure TProcess.ProcessPacket(lpacket:string);
//process a packet received from my ti
var
  id     : byte;
  i      : integer;
  data,a : string;
begin
  id:=ord(lpacket[3]);
  ack:=true; //send ack if not denied later on

  //ignore not-expected cids
  if not(id in [expectcid,cidemptypacket]+cidsetcontrolTI2Server) then
    exit;

  //check for multi packet
  if (id and 128)=128 then begin //cont'd multi packet
    if ord(lpacket[5]) <> mplastrcvd+1 then begin
      ack:=true;
      exit; //exit if we received the same part of the multi-packet twice
    end;
    mpbuffer:=mpbuffer+copy(lpacket,6,ord(lpacket[2])-6);
    if ord(lpacket[5])<mpnop then exit; //exit if not last packet received
    id:=id and 127;
    data:=mpbuffer;
  end else
    if ord(lpacket[5])>1 then begin //multi packet start
      mpbuffer   := copy(lpacket,6,ord(lpacket[2])-6);
      mpnop      := ord(lpacket[5]);
      mplastrcvd := 1;
      exit;
    end else
      data:=copy(lpacket,6,ord(lpacket[2])-6); //nothing with multi packet

  // process the content id of the packet we have just received
  case id of
    cidack:                //Empfangsbesttigung -> Paket aus Warteschleife lschen, da erfolgreich gesendet
      begin
        if length(packets)>0 then begin
          for i:=0 to high(packets)-1 do
            packets[i]:=packets[i+1];
          SetLength(packets,length(packets)-1);
        end;
        ack   :=false; //kein ACK
        resent:=0;     //reset resent
      end;
    cidnack:               //negative Empfangsbesttigung
      begin
        ack:=false;
      end;
    cidclientabort:        //Abbruch
      begin
        SetLength(packets,0);
        if not loggedin then //fixes security hole...
          Send2TILogInScreen
        else
        if lastcfile='' then
          Send2TIMainMenu
        else begin
          Send2TIContentFile(lastcfile);
          lastcfile:='';
        end;
      end;
    cidlogoutrequest:      //logout request
      begin
        TriggerUnregister:=true;
        ack:=false;
        AddPacket(cidlogoutack,''); //we just send an logout ack now
      end;
    cidemptypacket:        //leeres Paket -> nichts tun (auer: wenn Anmeldefreigabe in Queue, diese lschen)
      begin
        if length(packets)>0 then
          if packets[0].id=cidloginack then begin
            for i:=1 to high(packets) do
              packets[i-1]:=packets[i];
            SetLength(packets,length(packets)-1);
          end;
        ack       := false;
        rcvdempty := true;
      end;

    else                   //                        ---- Achtung ! ----
    if rcvdempty then begin
    rcvdempty:=false;
    case id of
    //hier kommen die mglichen Antwort-IDs
    cidreplyfullscreenmenu://Antwort auf full screen menu
      begin
        if (lpacket[6] in [TIkeyMenuR,TIkeyMenuL]) then //go left / right
          with longmenu do begin
            if lpacket[6]=TIkeyMenuR then begin
              inc(menup);
              if menup>menunop then menup:=0;
            end else begin
              dec(menup);
              if menup<0 then menup:=menunop;
            end;
            Send2TILongMenuPage(menup);
          end //of with
        else begin               //'1' - '7' : do action
           i:=ord(lpacket[6]);
           if (i<0) or ((i-1)>high(actions)) then begin
             Send2TIServerMessage('Diese Auswahl ist ungltig.','IDmenumain');
             exit;
          end;
          if i=0 then
            Send2TIMainMenu
          else
            ParseActions(Actions[i-1]);
        end;
      end;
    cidreplyplaintext:     //Antwort auf plain text, nchsten Teil des longtexts versenden
      begin
        if data[1]=TIkeyDown then begin //nchste Seite
          inc(longtextp);
          Send2TILongTextPage(longtextp);
        end else
        if data[1]=TIkeyUp then begin //vorherige Seite
          dec(longtextp);
          Send2TILongTextPage(longtextp);
        end else
        if (data[1]=TIkeyEnter) and (longtextn>0) then begin //Auswahl
          a:='Zu welcher Seite mchtest Du springen? ('+inttostr(longtextp+1)+' / 1-'+inttostr(longtextn+1)+')';
          AddPacket(cideditfield,#0#0+chr(length(a))+a+#4#0#0#9#9#0);
          okAct   :=MakeAction(0,'IDjumpToPage');
          breakAct:=Actions[0];
          storeto :='temp';
        end else
          ParseActions(Actions[0]);
      end;
    cidreplyeditfield:    //Antwort auf einzeiliges Eingabefeld
      begin
        if (ord(data[1]) and 1) = 0 then begin
          StoreToVar(copy(data,3,ord(data[2])));
          ParseActions(okAct);
        end else
          ParseActions(breakAct);
      end;
    else
      begin
        Log('unknown ID:'+inttostr(id)+' for ti '+inttostr(myaddr));
      end;
  end; //of second (inner) case
  end; //of if
  end; //of first (outer) case
end;
//
//             P R O C E S S   P A C K E T   F R O M   T I
//
////////////////////////////////////////////////////////////////////////////////


function TProcess.LoadFileToStr(var cfile:TAoS;filename:string):boolean;
//loads a file and returns its content as string;
var
  myfile : textfile;
  a      : string;
begin
  result:=false;
  if copy(filename,length(filename)-3,4)<>'.txt' then filename:=filename+'.txt';
  if not FileExists(filename) then exit;
  AssignFile(myfile,filename);
  ReSet(myfile);
  SetLength(cfile,0);
  while not eof(myfile) do begin
    ReadLn(myfile,a);
    SetLength(cfile,length(cfile)+1);
    cfile[high(cfile)]:=a;
  end;
  CloseFile(myfile);
  result:=true;
end;


////////////////////////////////////////////////////////////////////////////////
//
//                         P A R S E   A C T I O N S
//
procedure TProcess.ParseActions(action:Taction);
//process and queue a content file
var
  i,j     : integer;
  a       : string;  // dummy var
  daten   : string;  // content of packet to send to ti
  smsnummer,smstext,msgrcpt,temp:string;
begin
  case action.action of
    actContentFile:       // send content file
      begin
        Send2TIContentFile(action.cfile);
      end;
    actSendMail:          // send mail
      begin
        if datahash.GetValue('emailtext')='' then
          Send2TIServerMessage('Du hast noch keinen Text eingegeben.','entermailtext')
        else if datahash.GetValue('emailrcpt')='' then
          Send2TIServerMessage('Du hast noch keinen Empfnger eingegeben.','entermailtext')
        else begin
          Form1.MainLog(myuser+' sendet eine Mail an '+datahash.GetValue('emailrcpt'),False);
          SendMail;
        end;
      end;
    actGetMail:           // get mail
      GetMail;
    actPosteingang:       // Posteingang
      begin
        //full screen menu aus Posteingang zusammensetzen
        with LongMenu do begin
          SetLength(items,length(mails)+1);
          for i:=0 to high(mails) do begin
            if mails[i].msubj<>'' then
              items[high(mails)-i].title:=copy(mails[i].msubj,1,14)
            else
              items[high(mails)-i].title:='<kein Betreff>';
            items[high(mails)-i].action:=MakeAction(0,'IDShowMail');
            items[high(mails)-i].action.extinfo:=i;
          end;
          items[high(items)].title:='zurck';
          items[high(items)].action:=MakeAction(0,'menumail');
        end;
        Send2TILongMenu('E-Mails lesen',1);
      end;
    actShowMail:          // show mail
      begin
        i:=action.extinfo;
        if (i<0) or (i>high(mails)) then i:=0;
        shownmail:=i;
        with mails[shownmail] do
          Send2TIServerMessage('Mail '+inttostr(shownmail+1)+#10'von: '+mfrom+
                               #10'gesendet: '+mdate+#10'Betreff:'+msubj+
                               #10'Text:'#10+mbody,'menuaftershowmail');
      end;
    actReplyToMail:       // reply to mail
      begin
        if (shownmail<0) or (shownmail>high(mails)) then begin
          Send2TIServerError('Mail existiert nicht mehr.');
          exit;
        end;
        with mails[shownmail] do
          with datahash do begin
            SetValue('emailrcpt',mreply);
            SetValue('emailsubj','Re: '+msubj);
            SetValue('emailtext','');
          end;
        if datahash.GetValue('emailrcpt')='' then
          Send2TIServerMessage('Keine Antwortadresse.','entermailrcpt.txt')
        else
          Send2TIContentFile('entermailtext.txt');
      end;
    actNextMail:          // show next mail
      begin
        inc(shownmail);
        if (shownmail<0) then
          Send2TIServerError('Ungltige Mailnummer.');
        if shownmail>high(mails) then
          shownmail:=0;
        with mails[shownmail] do
          Send2TIServerMessage('Mail '+inttostr(shownmail)+#10'von: '+mfrom+
                               #10'gesendet: '+mdate+#10'Betreff:'+msubj+
                               #10'Text:'+mbody,'menuaftershowmail');
      end;
    actSendSMS:           // send sms
      begin
{$IFDEF SMSAvailable}
        smsnummer := datahash.GetValue('smsnummer');
        smstext   := datahash.GetValue('smstext');
        if smstext = '' then begin
          Send2TIServerMessage('Du hast noch keinen Text eingegeben.','menusms');
          exit;
        end;
        if (length(smsnummer)<11) or (length(smsnummer)>12) then begin
          Send2TIServerMessage('Die Nummer ist ungltig.','menusms');
          exit;
        end;
        for i:=1 to length(smsnummer) do
          if not(smsnummer[i] in ['0'..'9']) then begin
            Send2TIServerMessage('Die Nummer enthlt ein ungltiges Zeichen:'+smsnummer[i],'menusms');
            exit;
          end;
        //sending of sms is done in a thread because Get is used in sync mode (and would drown the server)
        if assigned(smsthread) then
          Send2TIServerMessage('Es wurde schon eine SMS versendet.','menusms')
        else begin
          Form1.MainLog(myuser+' versendet eine SMS an '+smsnummer+':'+smstext,true);
          smsthread:=TThreadSMS.Create(smstext,smsnummer,myuser,Send2TIServerMessage);
        end;
{$ELSE}
        Send2TIServerError('SMS konnte nicht gesendet werden: Option nicht verfgbar.');
{$ENDIF}
      end;
    actSendMessage:       // send message to other tinet user
      begin
        msgrcpt:=uppercase(datahash.GetValue('msgrcpt'));
        for i:=0 to high(userdata) do
          if userdata[i].uname=msgrcpt then begin
            with userdata[i] do begin
              SetLength(messages,length(messages)+1);
              with messages[high(messages)] do begin
                from:=myuser;
                msg :=datahash.GetValue('msgtext');
                time:=FormatDateTime('c',now);
                read:=false;
              end;
              if online then begin
                for j:=0 to high(TIT) do
                  if uppercase(TIT[j].myuser)=msgrcpt then begin
                    TIT[j].Send2TIServerMessage('Du hast Post.',TIT[j].lastcfile);
                    break;
                  end;
                Send2TIServerMessage('Die Nachricht wird an '+uname+' versendet.','menumessage');
              end else
                Send2TIServerMessage(uname+' ist nicht online. Die Nachricht wird gespeichert und bei nchster Gelegenheit gesendet.','menumessage');
              break;
            end;
          end;//of if
        if i>high(userdata) then
          Send2TIServerMessage('Der Benutzername '+msgrcpt+' existiert nicht.','menumessage');
      end;
    actListMessages:
      begin               // list messages
        //full screen menu aus Messages zusammensetzen
        if length(userdata[userdatai].messages)=0 then
          Send2TIServerMessage('Sie haben keine Nachrichten.','menumessage')
        else
          with LongMenu do begin
            SetLength(items,length(userdata[userdatai].messages)+1);
            for i:=0 to high(userdata[userdatai].messages) do begin
              items[i].title:=copy(userdata[userdatai].messages[i].msg,1,14);
              items[i].action:=MakeAction(0,'IDShowMessage');
              items[i].action.extinfo:=i;
            end;
            items[high(items)].title:='zurck';
            items[high(items)].action:=MakeAction(0,'menumessage');
            Send2TILongMenu('Messages lesen',1);
          end;
      end;
    actShowMessage:       // show message
      begin
        shownmsg:=action.extinfo;
        if (shownmsg<0) or (shownmsg>high(userdata[userdatai].messages)) then
          Send2TIServerError('Nachricht existiert nicht mehr.')
        else
          with userdata[userdatai].messages[shownmsg] do begin
            read:=true;
            Send2TIServerMessage('Nachricht '+inttostr(shownmsg+1)+' von '+from+' um '+time+': '+msg,'menuaftershowmessage');
          end;
      end;
    actDeleteMessage:     // delete currently shown message
      begin
        if (shownmsg<0) or (shownmsg>high(userdata[userdatai].messages)) then begin
          Send2TIServerError('Nachricht existiert nicht mehr.');
          exit;
        end;
        with userdata[userdatai] do begin
          for i:=shownmsg+1 to high(messages) do
            messages[i-1]:=messages[i];
          SetLength(messages,length(messages)-1);
        end;
        shownmsg:=-1;
        ParseActions(MakeAction(0,'IDListMessages'));
      end;
    actReplyToMessage:    // reply to currently shown message
      begin
        if (shownmsg<0) or (shownmsg>high(userdata[userdatai].messages)) then begin
          Send2TIServerError('Nachricht existiert nicht mehr.');
          exit;
        end;
        msgrcpt:=userdata[userdatai].messages[shownmsg].from;
        Send2TIContentFile('entermessagetext.txt');
      end;
    actMOTD:              // message of the day
      begin
        SetLength(actions,1);
        actions[0]:=MakeAction(0,'menuunterhaltung');
        Send2TILongText(WrapText(GetRandomMOTD));
      end;
    actLogin:             // try password for login
      begin
        if userdata[userdatai].upass<>datahash.GetValue('password') then
          Send2TIServerMessage('Das eingegebene Pawort konnte leider nicht besttigt werden.','login.txt')
        else begin
          // correctly logged-in
          loggedin:=true;
          case trunc(Time*24) of
            6..10 : daten:='Guten Morgen ';
            11..15: daten:='Guten Tag ';
            16..17: daten:='Guten Nachmittag ';
            18..20: daten:='Guten Abend ';
            21..23: daten:='Gute Nacht ';
            0..5  : daten:='Hallo Nachtschwrmer ';
          end;
          daten:=daten+myuser+'!'#10'Du hast ';
          //count new messages
          j:=0;
          for i:=0 to high(userdata[userdatai].messages) do
            if not userdata[userdatai].messages[i].read then
              inc(j);
          if j=0 then begin
            if Random>0.5 then
              daten:=daten+'keine ungelesene Nachricht.'
            else
              daten:=daten+'keine ungelesenen Nachrichten.';
          end
          else if j=1 then daten:=daten+' eine ungelesene Nachricht.'
          else daten:=daten+inttostr(j)+' ungelesene Nachrichten.';
          Send2TIServerMessage(daten,'IDmenumain');
          datahash.SetValue('password',''); //damit es beim Pawort ndern nicht angezeigt wird
        end;
      end;
    actLogout:            // logout
      begin
        TriggerUnregister:=true;
        ack:=false;
        AddPacket(8,'');
      end;
    actTime:              // send time
      begin
        Send2TIServerMessage('Heute ist der '+DateToStr(date)+'.'#10'Es ist jetzt '+copy(TimeToStr(time),1,5)+'.',lastcfile);
      end;
    actTranslateDE:       // translate Deutsch <-> English
      begin
        temp := datahash.GetValue('temp');
        if Translate(temp,a,TranslateDE) then
          Send2TIServerMessage(temp+#10'heit auf Englisch'#10+a,lastcfile)
        else
          Send2TIServerMessage(temp+' wurde leider nicht gefunden.',lastcfile);
      end;
    actTranslateED:       // translate English <-> Deutsch
      begin
        if Translate(temp,a,TranslateED) then
          Send2TIServerMessage(temp+#10'heit auf Deutsch'#10+a,lastcfile)
        else
          Send2TIServerMessage(temp+' wurde leider nicht gefunden.',lastcfile);
      end;
    actSendMainMenu:      // send main menu
      begin
        Send2TIMainMenu;
      end;
    actCreateUser:        // create new user
      begin
        if UserExists(temp) then begin
          Send2TIServerMessage('User '+temp+' already exists.','menuadmin');
          exit;
        end;
        SetLength(userdata,length(userdata)+1);
        with userdata[high(userdata)] do begin
          uname  := uppercase(temp);
          upass  := temp;
          admin  := false;
          online := false;
        end;
        Send2TIServerMessage('User '+uppercase(temp)+' was created.'#10'Password:'+temp,'menuadmin');
        Form1.MainLog('User '+temp+' was created by '+myuser,false);
      end;
    actListUsers:         // lists all users
      begin
        //full screen menu aus allen users zusammensetzen
        with LongMenu do begin
          SetLength(items,length(userdata)+1);
          for i:=0 to high(userdata) do begin
            items[i].title:=copy(userdata[i].uname,1,8);
            if userdata[i].online then
              items[i].title:=items[i].title+StringOfChar(' ',8-length(items[i].title))+' O';
            if userdata[userdatai].admin then
              items[i].action:=MakeAction(0,'IDEditUser')
            else
              items[i].action:=MakeAction(0,'IDmenumain');
            items[i].action.extinfo:=i;
          end;
          items[high(items)].title:='zurck';
          if userdata[userdatai].admin then
            items[high(items)].action:=MakeAction(0,'menuadminedituser')
          else
            items[high(items)].action:=MakeAction(0,lastcfile);
        end;
        Send2TILongMenu('List users',1);
      end;
    actEditUser:          // edit user
      begin
        shownuser:=action.extinfo;
        Send2TIContentFile('menuadminedituser');
      end;
    actKickUser:          // kick user
      begin
        if (shownuser<0) or (shownuser>high(userdata)) then
          Send2TIServerError('Nutzer existiert nicht.')
        else
          if userdata[shownuser].online then begin
            Form1.KickUser(shownuser);
            Send2TIServerMessage('User '+userdata[shownuser].uname+' wird gekickt.','menuadmin');
          end else
            Send2TIServerMessage('Nutzer '+userdata[shownuser].uname+' ist nicht online.','menuadmin')
      end;
    actJumpToPage:        // jump to page
      begin
        val(temp,i,i);
        if (i<1) or (i>longtextn+1) then
          Send2TIServerMessage('Ungltige Eingabe',lastcfile)
        else
          Send2TILongTextPage(i-1);
      end;
    actListFeedBack:      // list feed back
      begin
        //full screen menu aus allen users zusammensetzen
        with LongMenu do begin
          SetLength(items,length(Form1.feedback)+1);
          for i:=0 to high(Form1.feedback) do begin
            items[i].title:=copy(Form1.feedback[i].msg,1,14);
            items[i].action:=MakeAction(0,'IDShowFeedback');
            items[i].action.extinfo:=i;
          end;
          items[high(items)].title:='zurck';
          items[high(items)].action:=MakeAction(0,'menufeedback');
        end;
        Send2TILongMenu('Feedback lesen',1);
      end;
    actShowFeedback:      // show feed back
      begin
        i:=action.extinfo;
        if (i<0) or (i>high(Form1.feedback)) then
          Send2TIServerError('Feedback existiert nicht mehr.')
        else
          with Form1.feedback[i] do begin
            Send2TIServerMessage('Feedback '+inttostr(shownmsg+1)+' von '+from+' um '+time+':'#10+msg,'IDListFeedback');
          end;
      end;
    actSaveFeedback:      // save feed back
      begin
        SetLength(Form1.feedback,length(Form1.feedback)+1);
        with Form1.feedback[high(Form1.feedback)] do begin
          from:=myuser;
          msg :=temp;
          time:=FormatDateTime('c',now);
        end;
        Send2TIServerMessage('Vielen Dank fr Deinen Beitrag!','menufeedback');
      end;
    actChangePassword:    // change password
      begin
        if datahash.GetValue('password')<>datahash.GetValue('password2') then
          Send2TIServerMessage('Die beiden eingegebenen Pawrter stimmen nicht berein.','menukonfiguration')
        else begin
          userdata[userdatai].upass:=datahash.GetValue('password');
          Send2TIServerMessage('Das Pawort wurde gendert.','menukonfiguration');
        end;
      end;
    else
      Send2TIServerError('Diese Aktion wird noch nicht untersttzt.');
  end;//of case
end;
//
//                         P A R S E   A C T I O N S
//
////////////////////////////////////////////////////////////////////////////////


procedure TProcess.StoreToVar(data:string);
//stores the [data] in the variable the var storeto is referring to
begin
{  storeto:=uppercase(storeto);
  data:=TextTI2ASC(data);
  if storeto='EMAILTEXT' then emailtext:=data else
  if storeto='EMAILRCPT' then emailrcpt:=data else
  if storeto='EMAILSUBJ' then emailsubj:=data else
  if storeto='SMSTEXT'   then smstext  :=data else
  if storeto='SMSNUMMER' then smsnummer:=data else
  if storeto='MSGTEXT'   then msgtext  :=data else
  if storeto='MSGRCPT'   then msgrcpt  :=data else
  if storeto='PASSWORD'  then password :=data else
  if storeto='PASSWORD2' then password2:=data else
  if storeto='TEMP'      then temp     :=data else
  if storeto='POP3HOST'  then userdata[userdatai].pop3host:=data else
  if storeto='POP3USER'  then userdata[userdatai].pop3user:=data else
  if storeto='POP3PASS'  then userdata[userdatai].pop3pass:=data else
  if storeto='SMTPHOST'  then userdata[userdatai].SMTPhost:=data else
  if storeto='SMTPUSER'  then userdata[userdatai].SMTPuser:=data else
  if storeto='SMTPPASS'  then userdata[userdatai].SMTPpass:=data else
  if storeto='SMTPADDR'  then userdata[userdatai].SMTPaddr:=data else
  if storeto='SMTPNAME'  then userdata[userdatai].SMTPname:=data else
  Log('Unbekannte Variable:'+storeto);}
  datahash.SetValue(Uppercase(storeto),data);
end;


function TProcess.LoadFromVar(data:string):string;
//load the [data] into the variable the var storeto is referring to
begin
{  storeto:=uppercase(storeto);
  data:=TextASC2TI(data);
  if storeto='EMAILTEXT' then result:=emailtext else
  if storeto='EMAILRCPT' then result:=emailrcpt else
  if storeto='EMAILSUBJ' then result:=emailsubj else
  if storeto='SMSTEXT'   then result:=smstext   else
  if storeto='SMSNUMMER' then result:=smsnummer else
  if storeto='MSGTEXT'   then result:=msgtext   else
  if storeto='MSGRCPT'   then result:=msgrcpt   else
  if storeto='PASSWORD'  then result:=password  else
  if storeto='PASSWORD2' then result:=password2 else
  if storeto='TEMP'      then result:=''        else
  if storeto='POP3HOST'  then result:=userdata[userdatai].pop3host else
  if storeto='POP3USER'  then result:=userdata[userdatai].pop3user else
  if storeto='POP3PASS'  then result:=userdata[userdatai].pop3pass else
  if storeto='SMTPHOST'  then result:=userdata[userdatai].SMTPhost else
  if storeto='SMTPUSER'  then result:=userdata[userdatai].SMTPuser else
  if storeto='SMTPPASS'  then result:=userdata[userdatai].SMTPpass else
  if storeto='SMTPADDR'  then result:=userdata[userdatai].SMTPaddr else
  if storeto='SMTPNAME'  then result:=userdata[userdatai].SMTPname else
  Log('Unbekannte Variable:'+storeto);}
  result:=datahash.GetValue(Uppercase(data));
end;


procedure TProcess.Log(text:string);
begin
  logtext:=text;
  Form1.Log(logtext);
end;


function TProcess.MakeAction(fkeycode:integer;ffilename:string):Taction;
//return an action from the input
//replaces special IDs in var filename
var
  a : string;
begin
  ffilename:=lowercase(ffilename);
  with result do begin
    keycode:=fkeycode;
    if copy(ffilename,1,2)<>'id' then begin //normal content file
      action:=actContentFile;
      cfile:=ffilename;
    end else begin
      a:=copy(ffilename,3,50);
      if a='sendmail'    then action:=actSendMail else
      if a='getmail'     then action:=actGetMail else
      if a='showmail'    then action:=actShowMail else
      if a='mailnext'    then action:=actNextMail else
      if a='mailreply'   then action:=actReplyToMail else
      if a='sendsms'     then action:=actSendSMS else
      if a='posteingang' then action:=actPosteingang else
      if a='motd'        then action:=actMOTD else
      if a='sendmessage' then action:=actSendMessage else
      if a='listmessages'then action:=actListMessages else
      if a='showmessage' then action:=actShowMessage else
      if a='login'       then action:=actLogin else
      if a='logout'      then action:=actLogout else
      if a='time'        then action:=actTime else
      if a='transde'     then action:=actTranslateDE else
      if a='transed'     then action:=actTranslateED else
      if a='msgreply'    then action:=actReplyToMessage else
      if a='msgdelete'   then action:=actDeleteMessage else
      if a='menumain'    then action:=actSendMainMenu else
      if a='kickuser'    then action:=actKickUser else
      if a='createuser'  then action:=actCreateUser else
      if a='listusers'   then action:=actListUsers else
      if a='edituser'    then action:=actEditUser else
      if a='jumptopage'  then action:=actJumpToPage else
      if a='listfeedback'then action:=actListFeedBack else
      if a='showfeedback'then action:=actShowFeedBack else
      if a='savefeedback'then action:=actSaveFeedBack else
      if a='changepasswd'then action:=actChangePassword else
      Log('Unbekannte ID '+ffilename+' im Content-File gefunden.');
    end;
  end;
end;


////////////////////////////////////////////////////////////////////////////////
//
//                           S E N D   T O   T I
//
procedure TProcess.Send2TIMainMenu;
//sends main menu to ti
var
  a : string;
begin
  if userdata[userdatai].admin then
    a:=MainMenuAdminContentFile
  else
    a:=MainMenuContentFile;
  if FileExists(Form2.EditCSContent.Text+a) then begin
    lastcfile:=a;
    Send2TIContentFile(a)
  end else
    Send2TIServerError('Startmen nicht verfgbar.');
end;


procedure TProcess.Send2TIContentFile(filename:string);
//sends a content file to the ti
var
  cfile       : TAoS;    // = array of string, contains lines from file
  cid         : integer; // content id
  i,n,n1,d    : integer; // dummy vars
  daten,a     : string;
begin
  if not LoadFileToStr(cfile,Form2.EditCSContent.Text+filename) then begin
    Send2TIServerError('Fehler beim Laden des Content-Files '+Form2.EditCSContent.Text+filename);
    exit;
  end;
  daten:='';
  lastcfile:=filename;
  try
    val(cfile[1],cid,i);
    case cid of
      cidplaintext:         // plain text
        begin
           SetLength(actions,1);
           actions[0]:=MakeAction(0,cfile[2]);
           for i:=3 to high(cfile) do
             daten:=daten+cfile[i]+#10;
           Send2TILongText(WrapText(daten)); //send text
         end;
      cidfullscreenmenu:   // full screen menu
        begin
           with longmenu do begin
             n:=(high(cfile)-4) div 2;
             {val(cfile[2],n,i); //number of items
             if high(cfile)<(4+n*2) then begin
               Send2TIServerError('Content-File unvollstndig.');
               exit;
             end;}
             SetLength(items,n);
             val(cfile[3],d,i); //default item
             for i:=0 to n-1 do begin
               items[i].title :=copy(TextASC2TI(cfile[5+i*2]),1,TIScrWidthInChars[mytitype]-2);
               items[i].action:=MakeAction(ord('0')+i,(cfile[6+i*2]));
             end;
             Send2TILongMenu(copy(cfile[4],1,TIScrWidthInChars[mytitype]),d);
           end;
         end;
      cideditfield:        // edit field
        begin
           val(cfile[2],n,i);  //line no. of message text (column = 0)
           daten:=chr(n)+#0+chr(length(WrapText(cfile[4])))+WrapText(TextASC2TI(cfile[4]));
                                                            //chr(n)+#0 : Zeile/Spalte des und Aufforderungstext
                                                            //chr(length(cfile[4]))+cfile[4] : Lnge des und Aufforderungstext
           val(cfile[3],n,i);  //line no. of edit field (column = 0)
           daten:=daten+chr(n)+#0+#0;                       //chr(n)+#0 : Zeile/Spalte des Eingabefeldes
                                                            //#0        : erlaubte Zeichen (0 = alle)
           val(cfile[10],n,i); //width of edit field
           val(cfile[6],n1,i); //max. input length
           if n<3 then n:=DefaultWidthOfEditField else
             if n>MaxWidthOfEditField then n:=MaxWidthOfEditField;
           if n1<3 then n1:=16 else
             if n1>MaxLenOfEditField then n1:=MaxLenOfEditField;
           if n1<n then n1:=n;
           storeto  :=cfile[9];
           a:=LoadFromVar(storeto);
           if length(a)>240 then a:=copy(a,1,240);
           daten:=daten+chr(n)+chr(n1)+chr(length(a))+a;    //chr(n)    : angezeigte Breite des Eingabefeldes
                                                            //chr(n1)   : maximale Eingabelnge
                                                            //chr(length(a):Lnge des Vorgabetextes
                                                            //a         : Vorgabestring
           SetLength(actions,1);
           okAct    :=MakeAction(0,cfile[7]);
           breakAct :=MakeAction(0,cfile[8]);
           AddPacket(cid,daten);
         end;
    end; //of case
  except
    on e:exception do Send2TIServerError('Fehler im Content-File:'+e.Message);
  end;
end;


procedure TProcess.Send2TIServerError(text:string);
//send server error to user
begin
  Log('SERVER-ERR:'+text);
  SetLength(actions,1);
  actions[0]:=MakeAction(0,'IDmenumain');
  Send2TILongText(WrapText('Fehler im Server '+text));
end;


procedure TProcess.Send2TIServerMessage(text:string;nextstep:string);
//sends server message to user, nextstep is put into actions
begin
  SetLength(actions,1);
  actions[0]:=MakeAction(0,nextstep);
  Send2TILongText(Wraptext(text));
end;


procedure TProcess.Send2TILongText(text:String);
//sends a text longer than [TICharsPerScreen] chars to the TI
var
  data : string;
begin
  longtext:=text;
  if length(text)<=TICharsPerScreen[mytitype] then //calc number of pages
    longtextn:=0
  else
    longtextn:=((length(text)-1) div (TICharsPerScreen[mytitype]-TIScrWidthInChars[mytitype]));
  longtextp:=0;
  data:=TextASC2TI(copy(longtext,1,TICharsPerScreen[mytitype]))+#0;
  if longtextn>0 then //show arrow if user can scroll down
    data[length(data)-1]:=TIArrowDown;
  AddPacket(11,data);
end;


procedure TProcess.Send2TILongTextPage(pageno:integer);
//send a page of the long text to the ti
var
  data : string;
begin
  longtextp:=pageno;
  if longtextp<0 then
    longtextp:=longtextn
  else if longtextp>longtextn then
    longtextp:=0;
  data:=TextASC2TI(copy(longtext,longtextp*(TICharsPerScreen[mytitype]-TIScrWidthInChars[mytitype])+1,TICharsPerScreen[mytitype]))+#0;
  if longtextp>0 then  //show arrows if scrollable
    data[1]:=TIArrowUp;
  if longtextp<longtextn then
    data[length(data)-1]:=TIArrowDown;
  AddPacket(cidplaintext,data);
end;


procedure TProcess.Send2TILongMenu(menutitle:string;default:Integer);
//sends a menu with more than 7 (nearly unlimited) of items to the TI
begin
  with longmenu do begin
    menunop:=((length(items)-1) div 7);
    menup:=0;
    title:=menutitle;
    Send2TILongMenuPage(0);
  end;
end;


procedure TProcess.Send2TILongMenuPage(pageno:integer);
//send a page of a long menu to the ti
var
  a     : string;
  daten : string;
  n     : integer;
  i     : integer;
begin
  with longmenu do begin
    if (pageno>menunop) or (pageno<0) then begin
      Send2TIServerError('Menseite existiert nicht.');
      exit;
    end;
    menup:=pageno;
    a:=TextASC2TI(title)+StringOfChar(' ',TIScrWidthInChars[mytitype]-length(title));
    if menup>0 then            //add left arrow to show that user can go back
      a:=copy(TIArrowLeft+a,1,TIScrWidthInChars[mytitype]);
    if menup<menunop then begin//set n to number of items
      n:=7;
      a[TIScrWidthInChars[mytitype]]:=TIArrowRight; //add right arrow
    end else
      n:=((length(items)-1) mod 7)+1;
    daten:=chr(n)+chr(1)+#0+a+#0;
    SetLength(actions,n);
    for i:=0 to n-1 do begin
      daten:=daten+TextASC2TI(items[menup*7+i].title)+#0;
      actions[i]:=items[menup*7+i].action;
    end;
    AddPacket(14,daten);
  end;
end;


procedure TProcess.Send2TILogInScreen;
//send login mask to ti
begin
  if FileExists(Form2.EditCSContent.Text+'login.txt') then
    Send2TIContentFile('login.txt')
  else
    Send2TIServerError('Login nicht verfgbar.');
end;

//
//                           S E N D   T O   T I
//
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//
//                        T R A N S F O R M   T E X T
//
function TProcess.TextASC2TI(text:string):string;
//converts text from PC ASCII to TI ASCII, depending on mytitype
var
  i     : integer;
  table : TByteTable;
begin
  result:='';
  case mytitype of
    TI82 : table := tableASC2TI82;
    TI86 : table := tableASC2TI86;
  end;
  for i:=1 to length(text) do
    if table[ord(text[i])]<>0 then
      result:=result+chr(table[ord(text[i])])
    else
      result:=result+text[i];
end;


function TProcess.TextTI2ASC(text:string):string;
//converts text from TI ASCII to PC ASCII, depending on mytitype
var
  i     : integer;
  table : TByteTable;
begin
  result:='';
  case mytitype of
    TI82 : table := tableTI822ASC;
    TI86 : table := tableTI862ASC;
  end;
  for i:=1 to length(text) do
    if table[ord(text[i])]<>0 then
      result:=result+chr(table[ord(text[i])])
    else
      result:=result+text[i];
end;


function TProcess.WrapText(text:string):string;
//wraps the text to [width] chars by adding spaces (cool source, isn't it :)
var
  p,w      : word;
  a        : string;
begin
  if Form2.CheckCSNoWrap.Checked then begin
    result:=text;
    exit;
  end;
  w:=0;
  while length(text)>0 do begin
    p:=1;
    while not(text[p] in [' ',#10]) and (p<=length(text)) do inc(p);
    if p<=(TIScrWidthInChars[myTItype]-w) then begin     //word+space fit current line
      inc(w,p);
    end else if p=((TIScrWidthInChars[myTItype]+1)-w) then begin //word       fits current line
      w:=0;                                // ->delete space or #10
      delete(text,p,1);
      dec(p);
    end else begin                         //word(+space) don't fit current line
      if p>(TIScrWidthInChars[myTItype]+1) then begin    //word+space is longer than one line
        w:=(length(a)+p) mod TIScrWidthInChars[myTItype];
      end else if p=(TIScrWidthInChars[myTItype]+1) then begin //word fits one line
        delete(text,p,1);                  // ->delete space or #10
        dec(p);
        a:=a+stringofchar(' ',TIScrWidthInChars[myTItype]-w);
        w:=0;
      end else begin                       //word+space fit one (but not current) line
        a:=a+stringofchar(' ',TIScrWidthInChars[myTItype]-w);
        w:=p;
      end;
    end;
    if text[p]=#10 then begin              //add word to text a
      a:=a+copy(text,1,p-1);
      a:=a+StringOfChar(' ',(TIScrWidthInChars[myTItype]+1)-w);
      w:=0;
    end else
      a:=a+copy(text,1,p);
    delete(text,1,p);
  end;
  result:=a;
end;
//
//                        T R A N S F O R M   T E X T
//
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//
//                             S E N D   M A I L
//
procedure TProcess.SendMail;
//sends a mail with var emailtext to var emailrcpt
var
  MySmtp : TSmtpCli;
begin
  MySmtp:=TSmtpCli.Create(Application);
  with MySMTP do begin
    Host          := userdata[userdatai].smtphost;
    Username      := userdata[userdatai].smtpuser;
    Password      := userdata[userdatai].smtppass;
    Port          := 'smtp';
    AuthType      := smtpAuthLogin;
    OnRequestDone := SMTPRequestDone;
    try
      Open;
    except
      on E:Exception do
        Send2TIServerError('Exception on opening smtp connection'#10+E.message);
    end;
  end;
end;


procedure TProcess.SMTPRequestDone(Sender: TObject; RqType: TSmtpRequest; Error: Word);
//next step
begin
  with (Sender as TSmtpCli) do
  try
    if error>0 then begin
      Send2TIServerError('SMTP-Fehler:'+#10+LastResponse);
      Free;
      exit;
    end;
    case RqType of
      smtpOpen:
        begin
          with userdata[userdatai] do begin
            FromName   := smtpuser;
            HdrFrom    := smtpname;
            HdrReplyTo := smtpaddr;
          end;
          with datahash do begin
            HdrSubject := GetValue('emailsubj');
            HdrTo      := GetValue('emailrcpt');
            MailMessage.Add(GetValue('emailtext'));
            RcptName.Add(GetValue('emailrcpt'));
          end;
          Mail;
        end;
      smtpMail:
        Quit;
      smtpQuit:
        begin
          Send2TIServerMessage('Die Mail wurde erfolgreich versendet.','menumail');
          Free;
        end;
    end;
  except
    on e:exception do begin
      Send2TIServerError('Ausnahmefehler beim SMTP-Zugriff.'+e.message);
      Free;
    end;//of on
  end;//of except
end;


{procedure TForm1.SMTPResponse(Sender: TObject; Msg: String);
//show response from smtp component
begin
  Log(Msg);
end;}
//
//                             S E N D   M A I L
//
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//
//                        R E C E I V E    M A I L
//
procedure TProcess.GetMail;
//gets mail from an pop3 account and returns success/failure message
var
  MyPop3 : TPop3Cli;
begin
  MyPop3:=TPop3Cli.Create(Application);
  with MyPop3 do begin
    with userdata[userdatai] do begin
      Host          := Pop3Host;
      UserName      := Pop3User;
      PassWord      := Pop3Pass;
    end;
    Port          := 'pop3';
    OnRequestDone := Pop3RequestDone;
    OnListLine    := Pop3ListLine;
    OnMessageLine := Pop3MessageLine;
    try
      Open;
    except
      on E:Exception do
        Send2TIServerError('Exception on opening pop3 connection'#10+E.message);
    end;
  end;
end;


procedure TProcess.Pop3RequestDone(Sender: TObject; RqType: TPop3Request; Error: Word);
//next step
begin
  with (Sender as TPop3Cli) do
  try
    if error>0 then begin
      Send2TIServerError('POP3-Fehler:'+#10+LastResponse);
      Free;
      exit;
    end;
    case RqType of
      pop3Open:
        Stat;
      pop3Stat:
        List;
      pop3List:
        begin
          MsgNum:=MsgCount-Form1.MaxNumofMails;
          if MsgNum<1 then MsgNum:=1;
          SetLength(mails,MsgCount-MsgNum+1);
          MsgHeader:=true;
          mailnum  :=0;
          if MsgCount=0 then
            Quit
          else
            Retr;
        end;
      pop3Retr:
        begin
          MsgNum:=MsgNum+1;
          inc(mailnum);
          MsgHeader:=true;
          if Msgnum>MsgCount then
            Quit
          else
            Retr;
        end;
      pop3Quit:
        begin
          Send2TIServerMessage('Der POP3-Posteingang wurde erfolgreich abgerufen.','IDPosteingang');
          Free;
        end;
    end;//of case
  except
    on e:exception do begin
      Send2TIServerError('Ausnahmefehler beim POP3-Zugriff.'+e.message);
      Free;
    end;//of on
  end;//of except
end;


procedure TProcess.Pop3ListLine(Sender: TObject);
begin
  with (Sender as TPop3Cli) do begin
    Log('Mail '+inttostr(MsgNum)+' Lnge:'+inttostr(MsgSize));
  end;
end;


procedure TProcess.Pop3MessageLine(Sender: TObject);
begin
  with (Sender as TPop3Cli) do begin
    if not msgheader then
      mails[mailNum].mbody:=mails[mailNum].mbody+LastResponse
    else
      if LastResponse='' then
        msgheader:=false
      else begin
        if pos('Subject',LastResponse)>0 then
          mails[mailnum].msubj:=copy(LastResponse,pos(' ',LastResponse)+1,100)
        else if pos('From',LastResponse)>0 then
          mails[mailnum].mfrom:=copy(LastResponse,pos(' ',LastResponse)+1,100)
        else if pos('Date',LastResponse)>0 then
          mails[mailnum].mdate:=copy(LastResponse,pos(' ',LastResponse)+1,100)
        else if (pos('Return-Path',LastResponse)>0) or (pos('Reply-To',LastResponse)>0) then
          mails[mailnum].mreply:=copy(LastResponse,pos(' ',LastResponse)+1,100);
      end;
  end;
end;
//
//                        R E C E I V E    M A I L
//
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//
//                   M E S S A G E   O F   T H E   D A Y
//
function TProcess.GetRandomMOTD:string;
//returns a random motd
var
  motdfile   : TAoS;
begin
  if not LoadFileToStr(motdfile,Form2.EditCSContent.Text+MOTDRessourceFile) then begin
    Send2TIServerError('Fehler beim Laden von '+Form2.EditCSContent.Text+MOTDRessourceFile);
    exit;
  end;
  result:=motdfile[trunc(high(motdfile)*random)];
end;
//
//                   M E S S A G E   O F   T H E   D A Y
//
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//
//                           D I C T I O N A R Y
//
function TProcess.Translate(var word:string;var translation:string;direction:Translate):boolean;
//use dictionary
var
  myfile : textfile;
  a,b    : string;
begin
  result:=false;
  if not FileExists(Form2.EditCSContent.Text+DictionaryRessourceFile) then exit;
  Assignfile(myfile,Form2.EditCSContent.Text+DictionaryRessourceFile);
  ReSet(myfile);
  while not eof(myfile) do begin
    Readln(myfile,a);
    Readln(myfile,b);
    if direction=TranslateDE then begin
      if lowercase(a)=lowercase(word) then begin
        translation:=b;
        word:=a;
        result:=true;
        break;
      end
    end else begin
      if lowercase(b)=lowercase(word) then begin
        translation:=a;
        word:=b;
        result:=true;
        break;
      end;
    end;
  end;
  CloseFile(myfile);
end;
//
//                           D I C T I O N A R Y
//
////////////////////////////////////////////////////////////////////////////////



function TProcess.UserExists(name:string):boolean;
//returns true if this user name was found
var
  i     : integer;
begin
  name  :=uppercase(name);
  result:=false;
  for i:=0 to high(userdata) do
    if uppercase(userdata[i].uname)=name then begin
      result:=true;
      break;
    end;
end;

end.
