//modified:2002.07.16@22:05 by Alex
//Project: TI-Server v3

//defines all necessary (non-object) types and constants

unit UnitTypes;

interface

type
  TAoS      = array of string;
  TByteTable= array [0..255] of byte;
  TLoglevel = (llno,lllow,llnormal,llhigh,llhighest);
  TSendServerMessage = procedure (text:string;nextstep:string) of object;
  TactionID = (actContentFile,actSendMail,actGetMail,actSendSMS,actPosteingang,
               actMOTD,actShowMail,actSendMessage,actListMessages,actShowMessage,
               actLogin,actLogout,actTime,actTranslateDE,actTranslateED,actDeleteMessage,
               actReplyToMessage,actReplyToMail,actNextMail,actSendMainMenu,
               actKickUser,actCreateUser,actListUsers,actEditUser,actJumpToPage,
               actListFeedBack,actShowFeedback,actSaveFeedback,actChangePassword);
  Translate = (TranslateDE,TranslateED);
  TTXEN     = (TXsend,TXreceive);
  TTIType   = (TI82,TI86);

  TPacket   = record
    id      : byte;     //content-id
    mpinfo  : byte;     //multi packet info
    data    : string;
  end;

  TAction   = record
    keycode : byte;     // at the moment ASCII code!, 0 means any
    action  : TactionID;// action ID
    cfile   : string;   // name of content file to load
    extinfo : integer;  // extended info
  end;

  Tmessage  = record    // ti-net message
    from    : string;
    msg     : string;
    time    : string;
    read    : boolean;  // has user read this message?
  end;

  Tuserdata = record    // to store offline user data
    uname   : string;   // username
    upass   : string;   // password
    admin   : boolean;  // admin rights activated?
    pop3user: string;   //   .     .     (Anmeldename z.B. fuenfundachtzig)
    pop3host: string;   //   .     .
    pop3pass: string;   //   .     .
    smtpaddr: string;   //   .     .     (Antwortadresse z.B. fueufundachtzig@gmx.de)
    smtphost: string;   //   .     .
    smtpuser: string;   //   .     .     (Anmeldename z.B. fuenfundachtzig)
    smtppass: string;   //   .     .
    smtpname: string;   //   .     .     (Nutzername  z.B. Alexander Mann)
    messages: array of Tmessage;
    online  : boolean   // whether user is online
  end;

  TMail     = record     //Posteingang
    mbody   : string;    //mail body
    mfrom   : string;    //mail from
    mdate   : string;    //mail date
    msubj   : string;    //mail subject
    mreply  : string;    //mail reply address
  end;

  TLMenuItem = record    //ein Menpunkt des Langmens
    title   : string[14];
    action  : TAction;
  end;

  TLongMenu = record
    title   : string[16];//Titel des Mens
    menunop : integer;   //Anzahl der Seiten im Men
    menup   : integer;   //aktuelle Seite des Mens
    items   : array of TLMenuItem;
  end;

  TFeedback = record     //Nutzer-Rckmeldung
    from    : string;
    msg     : string;
    time    : string;
  end;

  THash     = class      //Delphi hasn't got built-in hashes
    protected
      values : array of string;
      names  : array of string;
    public
      constructor Create;
      destructor  Free;
      function  GetValue(n:string):string;
      procedure SetValue(n:string;v:string);
      function  GetIndexOf(n:string):integer;
  end;

const
  version     : string = 'Privat';
  loglevel    : TLogLevel = llhigh;
  LogLevelStr : array[TLoglevel] of string = ('no logging',
                                              'low logging',
                                              'normal logging',
                                              'high logging',
                                              'highest logging');
  //miscellaneous constants
  ServerAddress           : byte = 1;
  TICharsPerScreen        : array[TTIType]of byte=(128,168);//chars per TI screen (16*8,21*8)
  TIScrWidthInChars       : array[TTIType]of byte=(16,21);  //width of TI screen in chars
  TILogInCode             : array[TTIType]of byte=($82,$86);//code that the TI sends as its type
  TITypeAsString          : array[TTIType]of string=('TI82','TI86'); //ti type as string
  MaxLenOfEditField       : byte = 247;                     //maximum length of edit fields
  DefaultWidthOfEditField : byte = 16;                      //default width of edit fields
  MaxWidthOfEditField     : byte = 16;                      //maximum width of edit fields
  MainMenuContentFile     : string = 'menumain.txt';        //filename of content file containing the main menu
  MainMenuAdminContentFile: string = 'menumainadmin.txt';   //filename of content file containing the main menu for admins
  MOTDRessourceFile       : string = 'resmotd.txt';         //filename of the ressource file with the messages of the day
  DictionaryRessourceFile : string = 'resdict.txt';         //filename of dictionary file (Deutsch <> English)
  FeedbackRessourceFile   : string = 'resfdbck.txt';        //filename of feedback file

  //content ids
  //control
  cidloginrequest         = 0;
  cidack                  = 1;
  cidclientabort          = 2;
  cidlogoutrequest        = 3;
  cidsendefreigabe        = 4;
  cidnack                 = 5;
  cidanmeldefreigabe      = 6;
  cidloginack             = 7;
  cidlogoutack            = 8;
  cidnewaddr              = 9;
  //content
  cidemptypacket          = 10;
  cidplaintext            = 11;
  cidtiml                 = 12;
  cidscreenshot           = 13;
  cidfullscreenmenu       = 14;
  cideditfield            = 15;
  //reply
  cidreplyfullscreenmenu  = 32;
  cidreplyplaintext       = 33;
  cidreplyscreenshot      = 35;
  cidreplytiml            = 34;
  cidreplyeditfield       = 36;
  //sets
  cidsetcontrolTI2Server  = [cidloginrequest, cidack, cidclientabort, cidlogoutrequest, cidnack];
  cidsetcontrolServer2TI  = [cidack, cidsendefreigabe, cidnack, cidanmeldefreigabe, cidloginack, cidlogoutack, cidnewaddr];

  //Paketaufbau
  paRcpt                  = 1;
  paLength                = 2;
  paCid                   = 3;
  paSender                = 4;
  paMultiPacket           = 5;
  paDataStart             = 6;

  //TI key codes
  TIkeyLeft    = #2;
  TIkeyMenuL   = #252;
  TIkeyRight   = #3;
  TIkeyMenuR   = #253;                        //im Long Menu werden andere Links/Rechts-Codes verwendet
  TIkeyUp      = #4;
  TIkeyDown    = #1;
  TIkeyEnter   = #9;
  TIkeyClear   = #$F;

  //TI named char set (for TI82!)
  TIArrowUp    : char = #6;
  TIArrowDown  : char = #7;
  TIArrowLeft  : char = #207;
  TIArrowRight : char = #5;

  tableASC2TI82 : TByteTable = //wenn hier z.B. an Stelle 32 eine 40 steht, wird ein Leerzeichen als #40 an den TI gesendet; eine 0 heit, das keine Ersetzung stattfindet
        (000,000,000,000,000,000,000,000,000,000,032,000,000,000,000,000, //  0.. 15
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 16.. 31
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 32
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 48
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 64
         000,000,000,000,000,000,000,000,000,000,000,193,000,000,000,000, // 80
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 96
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, //112
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, //128
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, //144
         000,186,000,000,000,000,000,000,184,000,224,000,000,000,000,000, //160
         212,000,018,213,182,195,000,208,000,000,000,000,000,000,000,185, //176
         139,138,140,000,141,000,000,178,147,146,148,149,155,154,156,157, //192
         000,180,163,162,164,000,165,205,091,171,170,172,173,000,000,188, //208
         143,142,144,000,145,000,000,179,151,150,152,153,159,158,160,161, //224
         191,181,167,166,168,000,169,000,000,175,174,176,177,000,000,000);//240..255
  tableTI822ASC : TByteTable = //wenn hier z.B. an Stelle 40 eine 32 steht, wird ein vom TI als #40 empfangenes Zeichen in ein Leerzeichen umgewandelt; eine 0 heit, das keine Ersetzung stattfindet
        (000,000,000,000,000,062,000,000,000,000,000,000,000,000,000,000, //  0.. 15
         000,000,178,000,000,000,000,000,000,000,000,000,000,000,000,000, // 16
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 32
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 48
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 64
         000,000,000,000,000,000,000,000,000,000,000,216,000,000,000,000, // 80
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 96
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, //112
         000,000,000,000,000,000,000,000,000,000,193,192,194,196,225,224, //128
         226,228,201,200,202,203,233,232,234,235,205,204,206,207,237,236, //144
         238,239,211,210,212,214,243,242,244,246,218,217,219,220,250,249, //160
         251,252,199,231,209,241,180,096,168,191,161,000,223,000,000,240, //176
         000,091,000,181,000,000,000,000,000,000,000,000,000,215,000,060, //192
         183,000,000,178,176,179,000,000,000,000,000,000,000,000,000,000, //208
         170,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, //224
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000);//240..255

  tableASC2TI86 : TByteTable = //same as above, only for ti86
         (000,000,000,000,000,000,000,000,000,000,000,000,000,214,000,000,//  0.. 16
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 16.. 31
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 32.. 47
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 48
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 64
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 80
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, // 96
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000, //112
         000,000,000,000,000,206,000,000,000,000,000,207,000,000,000,000, //128..143
         000,000,000,000,000,000,000,000,000,000,000,006,000,000,000,000, //144..
         000,186,000,000,000,000,000,000,184,000,227,000,000,210,000,000,
         020,000,018,213,182,195,000,208,000,000,000,000,000,000,000,185,
         139,138,140,000,141,000,000,178,147,146,148,149,155,154,156,157,
         000,180,163,162,164,000,165,010,000,171,170,172,173,000,000,188,
         143,142,144,000,145,000,000,179,151,150,152,153,159,158,160,161,
         191,181,167,166,168,000,169,000,000,175,174,176,177,000,000,000);//240..255
  tableTI862ASC : TByteTable =
         (000,000,000,000,000,000,155,000,000,000,215,000,000,000,000,000,
         000,000,178,000,176,000,000,000,000,000,000,000,000,000,000,000,
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,061,
         048,049,050,051,052,053,054,055,056,057,193,192,194,196,225,224,
         226,228,201,200,202,203,233,232,234,235,205,204,206,207,237,236,
         238,239,211,210,212,214,243,242,244,246,218,217,219,220,250,249,
         251,252,199,231,209,241,180,096,168,191,161,000,223,000,000,240,
         000,000,000,181,000,000,000,000,000,000,000,000,000,215,133,139,
         183,047,173,178,176,179,013,000,092,000,000,000,000,000,000,000,
         000,000,000,170,095,000,000,170,000,000,043,000,000,061,000,000,
         000,000,000,000,000,000,000,000,000,000,000,000,000,000,000,000);

implementation

  uses Sysutils;

  constructor THash.Create;
  begin
    inherited Create;
  end;

  destructor  THash.Free;
  begin
    inherited Free;
  end;

  procedure THash.SetValue(n:string;v:string);
  //set key with name n to value v
  begin
    values[GetIndexOf(n)] := v;
  end;

  function  THash.GetValue(n:string):string;
  //returns value with name n
  begin
    result := values[GetIndexOf(n)];
  end;

  function  THash.GetIndexOf(n:string):integer;
  //if key with name n is not found, a new key is created
  var
    i     : integer;
  begin
    result:=-1;
    n := uppercase(n);
    for i:=low(names) to high(names) do
      if (names[i] = n) then begin
        result:=i;
        break;
      end;
    if result<0 then begin
      SetLength(names ,length(names)+1);
      SetLength(values,length(names));
      names [high(names)] := n;
      result              := high(names);
    end;
  end;

end.
